## This concern has +follow+ and +unfollow+ controller actions
## When you use it, be sure that +set_followable+ method is implemented in your controller, otherwise an fatal error will raise.
module ReviewableController
  extend ActiveSupport::Concern

  included do
    before_action :set_review, only: [:destroy, :rate]
  end

  # GET /v1/learning_objects/1/reviews
  # GET /v1/learning_objects/1/reviews.json
  def index
    render json: Review.where(reviewable: reviewable)
  end

  # POST /v1/learning_objects/1/reviews
  # POST /v1/learning_objects/1/reviews.json
  def create
    review = Review.where(user: current_user, reviewable: reviewable).new(review_params)

    if ::Reviews::CreateReviewService.new.execute review, params[:rates]
      render json: review, status: :created
    else
      render json: review.errors, status: :unprocessable_entity
    end
  end

  # DELETE /v1/learning_objects/1/reviews/2
  # DELETE /v1/learning_objects/1/reviews/2.json
  def destroy
    @review.destroy
    render nothing: true, status: :ok
  end

  # User can rate the review to approve or not
  # POST /v1/learning_objects/1/reviews/2/rate
  # POST /v1/learning_objects/1/reviews/2/rate.json
  def rate
    approves = 'true' == params[:approves] ? true : false
    rate = Rate.where(user: current_user, review: @review).first_or_initialize

    if rate.update(approves: approves)
      render json: rate, status: :ok
    else
      render json: rate.errors, status: :unprocessable_entity
    end
  end

  protected

  # Never trust parameters from the scary internet, only allow the white list through.
  def review_params
    params.require(:review).permit(:name, :description, :pros, :cons)
  end

  def reviewable
    raise NotImplementedError
  end

  private

  def set_review
    @review ||= Review.find(params[:id])
  end

end